import constants from 'crds-constants';
import CMSService from '../../core/services/CMS.service';

describe('CMSService', () => {
  let http,
      httpBackend,
      fixture;

  const endpoint = `${window.__env__['CRDS_CMS_ENDPOINT']}api`;

  let baseTime = new Date(2016, 9, 1); // set to 10/1/2016 - month appears to be 0 based index however
  let todaysDate = moment(baseTime).format('YYYY-MM-DD');

  beforeEach(inject(function($injector) {
    http        = $injector.get('$http');
    httpBackend = $injector.get('$httpBackend');
    fixture = new CMSService(http);
    fixture.todaysDate = todaysDate;
    jasmine.clock().mockDate(baseTime);
    
  }))

  afterEach(() => {
    httpBackend.verifyNoOutstandingExpectation();
    httpBackend.verifyNoOutstandingRequest();
    jasmine.clock().mockDate();
  });

  describe('Series', () => {
    it('gets the current series when the end date of the series has not passed', () => {

      let series = {
        "series": [
          {
            "id": 236,
            "title": "Together",
            "description": "<p class=\"ng-scope\"><span id=\"docs-internal-guid-63f12df6-2cd6-144b-b591-c6bb136663ed\">Spend more time together<\/span><\/p><p><span>\u00a0<\/span><\/p>",
            "startDate": "2016-09-30 11:30:00",
            "endDate": "2016-10-02 12:30:00",
            "trailerLink": "https:\/\/www.youtube.com\/watch?v=NYJ8c81AQJw",
            "version": "1",
            "messages": [
              {
                "id": 3803,
                "title": "Why we are better together",
                "description": "<p><span>Life is better Together.<\/span><\/p>",
                "date": moment(baseTime).add(2, 'days').format('YYYY-MM-DD'),
                "series": 236
              }
            ],
            "created": moment().subtract(7, 'days'),
            "className": "Series"
          },
          {
            "id": 235,
            "title": "Business of Success",
            "description": "<p class=\"ng-scope\"><span id=\"docs-internal-guid-63f12df6-2cd6-144b-b591-c6bb136663ed\">Get business. Get success<\/span><\/p><p><span>\u00a0<\/span><\/p>",
            "startDate": "2016-09-01 11:30:00",
            "endDate": "2016-09-02 12:30:00",
            "trailerLink": "https:\/\/www.youtube.com\/watch?v=NYJ8c81AQJw",
            "version": "1",
            "messages": [
              {
                "id": 3802,
                "title": "Find you business",
                "description": "<p><span>Find business opportunities all around you<\/span><\/p>",
                "date": moment(baseTime).subtract(14, 'days').format('YYYY-MM-DD'),
                "series": 235
              }
            ],
            "created": moment(baseTime).subtract(1, 'months'),
            "className": "Series"
          },
          {
            "id": 234,
            "title": "Following Christ",
            "description": "<p class=\"ng-scope\"><span id=\"docs-internal-guid-63f12df6-2cd6-144b-b591-c6bb136663ed\">Ways to follow Christ in everyday life<\/span><\/p><p><span>\u00a0<\/span><\/p>",
            "startDate": "2016-7-08 11:30:00",
            "endDate": "2016-7-08 12:30:00",
            "trailerLink": "https:\/\/www.youtube.com\/watch?v=NYJ8c81AQJw",
            "version": "1",
            "messages": [
              {
                "id": 3801,
                "title": "Christlike Attributes",
                "description": "<p><span>Ways to better emulate Christ in your everyday life<\/span><\/p>",
                "date": moment(baseTime).subtract(2, 'months').format('YYYY-MM-DD'),
                "series": 234
              }
            ],
            "created": moment(baseTime).subtract(3, 'months'),
            "className": "Series"
          }
        ]
      }

      fixture.getCurrentSeries().then(response => {
        expect(response.title).toBe('Together')
      });
      httpBackend.expectGET(`${endpoint}/series?endDate__GreaterThanOrEqual=${todaysDate}&endDate__sort=ASC`).respond(200, series);
      httpBackend.flush();
    })

    it('returns undefined when there is no current series', () => {
      // If an array of Series objects is provided in this test, then the mocked GET request will always return a response,
      // even though that is not the behavior when you submit a GET request to the CMS and there is no current series.
      // When there is no current series, then the response should be an empty array.
      let series = {"series": []}
      fixture.getCurrentSeries().then(response => {
        expect(response).toBe(undefined)
      });
      httpBackend.expectGET(`${endpoint}/series?endDate__GreaterThanOrEqual=${todaysDate}&endDate__sort=ASC`).respond(200, series);
      httpBackend.flush();
    })

    it('should use HTTP call to obtain nearest series', () => {
      let series = {"series":[{"id":318,"title":"Series A","description":"<p><span>asdf2</span></p>","startDate":"2016-08-30","endDate":"2016-08-31","trailerLink":null,"version":"5","messages":[{"id":3879,"title":"test1","description":"<p>test1</p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3881,"title":"test2","description":"<p>test2</p>","date":"2016-09-14","version":"1","series":318,"created":"2016-09-14T17:08:34-04:00","className":"Message"},{"id":3882,"title":"test3","description":"<p>test3</p>","date":"2016-12-14","version":"1","series":318,"created":"2016-09-14T17:22:50-04:00","className":"Message"}],"created":"2016-08-30T11:35:43-04:00","className":"Series"}]}
      let fixture = new CMSService(http);
      fixture.getNearestSeries().then(response => {
        expect(response.title).toBe('Series A')
      });
      httpBackend.expectGET(`${endpoint}/series?startDate__GreaterThanOrEqual=${todaysDate}&startDate__sort=ASC&__limit[]=1`).respond(200, series);
      httpBackend.flush();
    });
    
    it('should use HTTP call to obtain last series', () => {
      let lastSeries = {"series":[{"id":318,"title":"Series A","description":"<p><span>asdf2</span></p>","startDate":"2016-08-30","endDate":"2016-08-31","trailerLink":null,"version":"5","messages":[{"id":3879,"title":"test1","description":"<p>test1</p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3881,"title":"test2","description":"<p>test2</p>","date":"2016-09-14","version":"1","series":318,"created":"2016-09-14T17:08:34-04:00","className":"Message"},{"id":3882,"title":"test3","description":"<p>test3</p>","date":"2016-12-14","version":"1","series":318,"created":"2016-09-14T17:22:50-04:00","className":"Message"}],"created":"2016-08-30T11:35:43-04:00","className":"Series"}]}
      let fixture = new CMSService(http);
      fixture.getLastSeries();
      httpBackend.expectGET(`${endpoint}/series?endDate__LessThanOrEqual=${todaysDate}&endDate__sort=DESC&__limit[]=1`).respond(200, lastSeries);
      httpBackend.flush();
    });

    it('should use HTTP call to obtain a series', () => {
      let query = 'id=1';
      let series = {"series":[{"id":1,"title":"Where I Went In '97...","description":"<p>Brian Tome's annual year-end personal assessment message.<\/p>","startDate":"1998-01-04","endDate":"1900-01-01","trailerLink":null,"version":"1","image":{"id":10491,"name":"DefaultVideoImage.jpg","title":"DefaultVideoImage","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/DefaultVideoImage.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"72241","cloudMetaJson":"{\"Dimensions\":\"600x330\",\"LastPut\":1463414963}","parent":817,"owner":1,"created":"2016-01-25T19:44:08-05:00","className":"CloudImage"},"messages":[{"id":454,"title":"Where I Went In '97... ","description":"<p>Test<\/p>","date":"1998-01-04","version":"1","series":1,"messageVideo":{"id":0,"containsAdultContent":null,"serviceId":null,"sourcePath":null,"className":"MessageVideo"},"messageAudio":{"id":6,"containsAdultContent":"0","serviceId":"find-destination","sourcePath":null,"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2007-08-11T00:00:00-04:00","className":"MessageAudio"},"tags":[{"id":28,"title":"-","messages":[454],"created":"2015-09-15T09:21:00-04:00","className":"Tag"}],"className":"Message"},{"id":3275,"title":"Where I Went In '97... ","description":"","date":"1998-01-04","version":"1","series":1,"className":"Message"}],"tags":[{"id":1,"title":"assets","series":[7,1],"created":"2015-07-30T15:58:55-04:00","className":"Tag"}],"created":"2016-01-25T19:44:08-05:00","className":"Series"}]};
      fixture.getSeries(query).then(response => {
        expect(response.length).toBe(1);
        expect(_.first(response).title).toBe("Where I Went In '97...");
      });
      httpBackend.expectGET(`${endpoint}/series?${query}`).respond(200, series);
      httpBackend.flush();
    })
  });

  describe('Features', () => {
    it('should use HTTP to obtain features', () => {
      let features = {"features":[{"id":47,"title":"A very long story about a girl name Jane","description":"<p><span>Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.<\/span><\/p>","status":null,"link":"https:\/\/crossroads.net","version":"5","image":{"id":14038,"name":"Untitled-3.png","title":"Untitled 3","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/Uploads\/Untitled-3.png","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"100220","cloudMetaJson":"{\"Dimensions\":\"291x193\",\"LastPut\":1471292281}","parent":602,"owner":78,"derivedImages":[1000,1011],"created":"2016-08-15T16:18:00-04:00","className":"CloudImage"},"section":1,"created":"2016-08-25T15:26:33-04:00","className":"Feature"},{"id":48,"title":"Go Big","description":"<p>A short message.<\/p>","status":null,"link":null,"version":"5","image":{"id":10482,"name":"1-16-15-bt-still.jpg","title":"1 16 15 bt still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/1-16-15-bt-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"51495","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1453580111}","parent":817,"owner":1,"derivedImages":[623,624,625,1001],"created":"2016-01-23T15:15:11-05:00","className":"CloudImage"},"section":1,"created":"2016-08-25T15:32:13-04:00","className":"Feature"},{"id":49,"title":"Get engaged Today","description":"<p><span>Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.<\/span><\/p>","status":null,"link":null,"version":"4","image":{"id":10672,"name":"190x110_GoForth.jpg","title":"190x110_GoForth","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GoForth.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"89055","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414760}","parent":817,"owner":1,"derivedImages":[1048],"created":"2016-01-25T19:44:39-05:00","className":"CloudImage"},"section":1,"created":"2016-08-25T15:33:15-04:00","className":"Feature"},{"id":50,"title":"Give Now","description":"<p>You should give, it will help.<\/p>","status":null,"link":"https:\/\/crossroads.net\/give","version":"3","image":{"id":10467,"name":"anywhere.jpg","title":"anywhere","filename":"https:\/\/crds-cms-uploads.imgix.net\/content\/images\/anywhere.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"28570","cloudMetaJson":"{\"Dimensions\":\"444x290\",\"LastPut\":1453513668}","parent":793,"owner":1,"derivedImages":[596,597,598,664,1049],"created":"2016-01-22T20:47:47-05:00","className":"CloudImage"},"section":2,"created":"2016-08-25T15:35:47-04:00","className":"Feature"},{"id":51,"title":"Group Up","description":"<p><span>Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.<\/span><\/p><p><a class=\"btn btn-standard\" href=\"http:\/\/www.crossroads.net\"><span>See How<\/span><\/a><\/p>","status":null,"link":null,"version":"5","image":{"id":866,"name":"baptism.png","title":"baptism","filename":"https:\/\/crds-cms-uploads.imgix.net\/content\/images\/baptism.png","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"213818","cloudMetaJson":"{\"Dimensions\":\"395x263\",\"LastPut\":1451872993}","parent":793,"owner":1,"derivedImages":[428,432,466,1050],"created":"2016-01-03T21:03:13-05:00","className":"CloudImage"},"section":2,"created":"2016-08-25T15:42:48-04:00","className":"Feature"},{"id":60,"title":"Stream Analytics Test","description":"<p><iframe width=\"640\" height=\"360\" src=\"https:\/\/player2.streamspot.com\/?playerId=2887fba1\" frameborder=\"0\" allowfullscreen=\"\"><\/iframe><\/p>","status":null,"link":null,"version":"2","image":{"id":14025,"name":"1-16-15-bt-still.jpg","title":"1 16 15 bt still","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/Uploads\/1-16-15-bt-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"51495","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1467988873}","parent":602,"owner":28,"derivedImages":[946,947,955,999],"created":"2016-07-08T10:41:13-04:00","className":"CloudImage"},"section":2,"created":"2016-09-01T18:18:52-04:00","className":"Feature"}]};
      fixture.getDigitalProgram().then((response) => {
        expect(response.length).toBe(6)
      });
      httpBackend.expectGET(`${endpoint}/features`).respond(200, features);
      httpBackend.flush();
    });
  })

  describe('Content Blocks', () => {
    it('should use HTTP to obtain a content block', () => {
      let block = {"contentblocks":[{"id":1,"title":"generalError","content":"<p><strong>Oh no!<\/strong> Looks like there's an error. Please fix and try again.<\/p>","type":"error","category":"common","className":"ContentBlock"}]};
      let query = 'id=1'
      let fixture = new CMSService(http);

      fixture.getContentBlock(query).then((result) => {
        expect(result.title).toBe('generalError');
      })
      httpBackend.expectGET(`${endpoint}/contentblock?${query}`).respond(200, block);
      httpBackend.flush();
    })
  })

  describe('Messages', () => {
    it('should use HTTP to obtain messages', () => {
      let messages = {"messages":[{"id":3646,"title":"Last Wednesday - Florence","description":"Last Wednesday - Florence","date":"2014-05-28","version":"1","series":{"id":"205","title":"The New Man","description":"<p>Jesus was the beginning of a new breed of men. Men who take a stand, even if no one else stands with them; vigilant men who faithfully protect; men who follow, then lead; and men who choose sacrifice in the name of love.<\/p>","startDate":"2016-04-12","endDate":"2016-05-01","trailerLink":null},"messageVideo":{"id":5918,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"still":{"id":13607,"name":"082813_LW_Florence_Still.jpg","title":"082813_LW_Florence_Still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/stills\/082813_LW_Florence_Still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":817,"owner":1,"derivedImages":[690],"created":"2013-08-30T10:55:17-04:00","className":"CloudImage"},"created":"2016-08-30T11:58:39-04:00","className":"MessageVideo"},"messageAudio":{"id":5384,"containsAdultContent":"0","serviceId":"last-wednesday-may-1","sourcePath":null,"source":{"id":12821,"name":"022614-lw-florence.mp3","title":"022614-lw-florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/022614-lw-florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:24-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2014-06-10T16:52:14-04:00","className":"MessageAudio"},"created":"2014-06-19T17:11:24-04:00","className":"Message","number":7,"sequence":[{"id":3635,"date":"2014-04-26","number":1,"created":"2014-04-28 15:08:51","title":"5 Marks of a Man"},{"id":3636,"date":"2014-05-03","number":2,"created":"2014-05-05 10:41:12","title":"Minority Position"},{"id":3637,"date":"2014-05-10","number":3,"created":"2014-06-19 16:55:03","title":"Work"},{"id":3638,"date":"2014-05-17","number":4,"created":"2014-05-19 12:30:15","title":"Team Player"},{"id":3639,"date":"2014-05-24","number":5,"created":"2014-06-05 12:41:54","title":"Protector "},{"id":3645,"date":"2014-05-28","number":6,"created":"2014-06-10 15:50:41","title":"Last Wednesday - Oakley"},{"id":3646,"date":"2014-05-28","number":7,"created":"2014-06-19 17:11:24","title":"Last Wednesday - Florence"},{"id":3640,"date":"2016-08-30","number":8,"created":"2014-06-10 15:44:07","title":"Vision"}],"combinedTags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"relatedMessages":[{"id":3879,"title":"test1","description":"<p>test1<\/p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3640,"title":"Vision","description":"<p>Brian Tome talks about how the new man has vision.<\/p>","date":"2016-08-30","version":"1","series":205,"program":{"id":12793,"name":"05_31-06_01_14Program.pdf","title":"05_31-06_01_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_31-06_01_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5916,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"source":{"id":12841,"name":"071914_Florence.mp4","title":"071914_Florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/071914_Florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:28-05:00","className":"CloudFile"},"created":"2016-08-30T10:08:17-04:00","className":"MessageVideo"},"messageAudio":{"id":5373,"containsAdultContent":"0","serviceId":"vision","sourcePath":null,"source":{"id":12796,"name":"thenewman_06.mp3","title":"thenewman_06","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_06.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudFile"},"still":{"id":12797,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudImage"},"created":"2014-06-10T15:44:07-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2014-06-10T15:44:07-04:00","className":"Message"},{"id":3588,"title":"About the Small Places","description":"<p>Program - Week 5 God is_____.<\/p>","date":"2013-08-30","version":"1","series":194,"program":{"id":12574,"name":"08_10-11_13Program_LO.pdf","title":"08_10-11_13Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/08_10-11_13Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:54:41-05:00","className":"CloudFile"},"messageVideo":{"id":5274,"containsAdultContent":"0","serviceId":"rjncKOup7NE","sourcePath":null,"source":{"id":12572,"name":"god_is_05.mp4","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/god_is_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12573,"name":"god_is_05_still.jpg","title":"god_is_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/god_is_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"66970","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463415014}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:36:19-04:00","className":"MessageVideo"},"messageAudio":{"id":5273,"containsAdultContent":"0","serviceId":"about-the-small-places","sourcePath":null,"source":{"id":12570,"name":"god_is_05.mp3","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/god_is_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12571,"name":"190x110_GodIs.jpg","title":"190x110_GodIs","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GodIs.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"58329","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414758}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:29:04-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-08-12T13:39:27-04:00","className":"Message"},{"id":3579,"title":"Last Wednesday","description":"<p>Last Wednesday 6.26<\/p>","date":"2013-06-26","version":"1","series":193,"program":{"id":13991,"name":"pdf-sample-1.pdf","title":"pdf sample 1","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/pdf-sample-1.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"7945","cloudMetaJson":"{\"LastPut\":1459449748}","parent":812,"owner":33,"created":"2016-03-31T14:42:27-04:00","className":"CloudFile"},"messageVideo":{"id":5256,"containsAdultContent":"0","serviceId":"yfaCi6N3jLE","sourcePath":null,"source":{"id":12537,"name":"062613-LW.mp4","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/062613-LW.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12538,"name":"062613-LW-still.jpg","title":"062613-LW-still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/062613-LW-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"88181","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463414721}","parent":833,"owner":1,"derivedImages":[930],"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-27T15:43:23-04:00","className":"MessageVideo"},"messageAudio":{"id":5257,"containsAdultContent":"0","serviceId":"last-wednesday-6-26","sourcePath":null,"source":{"id":12539,"name":"062613-LW.mp3","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/062613-LW.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-27T15:40:39-04:00","className":"MessageAudio"},"speakers":[{"id":1,"title":"Brian Tome","messages":[3579,3265],"created":"2015-08-07T11:07:21-04:00","className":"Speaker"}],"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-27T15:40:39-04:00","className":"Message"},{"id":3578,"title":"Building Spiritual Maturity - Florence","description":"<p>Terry Phillips talks about building spiritual maturity.<\/p>","date":"2013-06-15","version":"1","series":193,"messageVideo":{"id":5255,"containsAdultContent":"0","serviceId":"2P8Xzyh-mGE","sourcePath":null,"source":{"id":12535,"name":"htbp_01_florence.mp4","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/htbp_01_florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12536,"name":"htbp_01_still_fl.jpg","title":"htbp_01_still_fl","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/htbp_01_still_fl.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-21T14:39:38-04:00","className":"MessageVideo"},"messageAudio":{"id":5254,"containsAdultContent":"0","serviceId":"building-spiritual-maturity-1","sourcePath":null,"source":{"id":12534,"name":"htbp_01_florence.mp3","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/htbp_01_florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:34-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-21T14:36:24-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-21T14:39:38-04:00","className":"Message"}]},{"id":3645,"title":"Last Wednesday - Oakley","description":"Last Wednesday - Oakley","date":"2014-05-28","version":"1","series":{"id":"205","title":"The New Man","description":"<p>Jesus was the beginning of a new breed of men. Men who take a stand, even if no one else stands with them; vigilant men who faithfully protect; men who follow, then lead; and men who choose sacrifice in the name of love.<\/p>","startDate":"2016-04-12","endDate":"2016-05-01","trailerLink":null},"messageVideo":{"id":5383,"containsAdultContent":"0","serviceId":"c41wQrbdDhM","sourcePath":null,"source":{"id":12819,"name":"052814-lw-oakley.mp4","title":"052814-lw-oakley","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/052814-lw-oakley.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:23-05:00","className":"CloudFile"},"still":{"id":12820,"name":"last-wednesday.jpg","title":"last-wednesday","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/last-wednesday.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:23-05:00","className":"CloudImage"},"created":"2014-06-10T15:50:41-04:00","className":"MessageVideo"},"messageAudio":{"id":5382,"containsAdultContent":"0","serviceId":"last-wednesday-may","sourcePath":null,"source":{"id":12818,"name":"052814-lw-oakley.mp3","title":"052814-lw-oakley","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/052814-lw-oakley.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:23-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2014-06-10T15:39:36-04:00","className":"MessageAudio"},"created":"2014-06-10T15:50:41-04:00","className":"Message","number":6,"sequence":[{"id":3635,"date":"2014-04-26","number":1,"created":"2014-04-28 15:08:51","title":"5 Marks of a Man"},{"id":3636,"date":"2014-05-03","number":2,"created":"2014-05-05 10:41:12","title":"Minority Position"},{"id":3637,"date":"2014-05-10","number":3,"created":"2014-06-19 16:55:03","title":"Work"},{"id":3638,"date":"2014-05-17","number":4,"created":"2014-05-19 12:30:15","title":"Team Player"},{"id":3639,"date":"2014-05-24","number":5,"created":"2014-06-05 12:41:54","title":"Protector "},{"id":3645,"date":"2014-05-28","number":6,"created":"2014-06-10 15:50:41","title":"Last Wednesday - Oakley"},{"id":3646,"date":"2014-05-28","number":7,"created":"2014-06-19 17:11:24","title":"Last Wednesday - Florence"},{"id":3640,"date":"2016-08-30","number":8,"created":"2014-06-10 15:44:07","title":"Vision"}],"combinedTags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"relatedMessages":[{"id":3879,"title":"test1","description":"<p>test1<\/p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3640,"title":"Vision","description":"<p>Brian Tome talks about how the new man has vision.<\/p>","date":"2016-08-30","version":"1","series":205,"program":{"id":12793,"name":"05_31-06_01_14Program.pdf","title":"05_31-06_01_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_31-06_01_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5916,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"source":{"id":12841,"name":"071914_Florence.mp4","title":"071914_Florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/071914_Florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:28-05:00","className":"CloudFile"},"created":"2016-08-30T10:08:17-04:00","className":"MessageVideo"},"messageAudio":{"id":5373,"containsAdultContent":"0","serviceId":"vision","sourcePath":null,"source":{"id":12796,"name":"thenewman_06.mp3","title":"thenewman_06","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_06.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudFile"},"still":{"id":12797,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudImage"},"created":"2014-06-10T15:44:07-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2014-06-10T15:44:07-04:00","className":"Message"},{"id":3588,"title":"About the Small Places","description":"<p>Program - Week 5 God is_____.<\/p>","date":"2013-08-30","version":"1","series":194,"program":{"id":12574,"name":"08_10-11_13Program_LO.pdf","title":"08_10-11_13Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/08_10-11_13Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:54:41-05:00","className":"CloudFile"},"messageVideo":{"id":5274,"containsAdultContent":"0","serviceId":"rjncKOup7NE","sourcePath":null,"source":{"id":12572,"name":"god_is_05.mp4","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/god_is_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12573,"name":"god_is_05_still.jpg","title":"god_is_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/god_is_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"66970","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463415014}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:36:19-04:00","className":"MessageVideo"},"messageAudio":{"id":5273,"containsAdultContent":"0","serviceId":"about-the-small-places","sourcePath":null,"source":{"id":12570,"name":"god_is_05.mp3","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/god_is_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12571,"name":"190x110_GodIs.jpg","title":"190x110_GodIs","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GodIs.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"58329","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414758}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:29:04-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-08-12T13:39:27-04:00","className":"Message"},{"id":3579,"title":"Last Wednesday","description":"<p>Last Wednesday 6.26<\/p>","date":"2013-06-26","version":"1","series":193,"program":{"id":13991,"name":"pdf-sample-1.pdf","title":"pdf sample 1","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/pdf-sample-1.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"7945","cloudMetaJson":"{\"LastPut\":1459449748}","parent":812,"owner":33,"created":"2016-03-31T14:42:27-04:00","className":"CloudFile"},"messageVideo":{"id":5256,"containsAdultContent":"0","serviceId":"yfaCi6N3jLE","sourcePath":null,"source":{"id":12537,"name":"062613-LW.mp4","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/062613-LW.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12538,"name":"062613-LW-still.jpg","title":"062613-LW-still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/062613-LW-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"88181","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463414721}","parent":833,"owner":1,"derivedImages":[930],"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-27T15:43:23-04:00","className":"MessageVideo"},"messageAudio":{"id":5257,"containsAdultContent":"0","serviceId":"last-wednesday-6-26","sourcePath":null,"source":{"id":12539,"name":"062613-LW.mp3","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/062613-LW.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-27T15:40:39-04:00","className":"MessageAudio"},"speakers":[{"id":1,"title":"Brian Tome","messages":[3579,3265],"created":"2015-08-07T11:07:21-04:00","className":"Speaker"}],"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-27T15:40:39-04:00","className":"Message"},{"id":3578,"title":"Building Spiritual Maturity - Florence","description":"<p>Terry Phillips talks about building spiritual maturity.<\/p>","date":"2013-06-15","version":"1","series":193,"messageVideo":{"id":5255,"containsAdultContent":"0","serviceId":"2P8Xzyh-mGE","sourcePath":null,"source":{"id":12535,"name":"htbp_01_florence.mp4","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/htbp_01_florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12536,"name":"htbp_01_still_fl.jpg","title":"htbp_01_still_fl","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/htbp_01_still_fl.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-21T14:39:38-04:00","className":"MessageVideo"},"messageAudio":{"id":5254,"containsAdultContent":"0","serviceId":"building-spiritual-maturity-1","sourcePath":null,"source":{"id":12534,"name":"htbp_01_florence.mp3","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/htbp_01_florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:34-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-21T14:36:24-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-21T14:39:38-04:00","className":"Message"}]},{"id":3639,"title":"Protector ","description":"Brian Tome talks about how the new man is a protector.","date":"2014-05-24","version":"1","series":{"id":"205","title":"The New Man","description":"<p>Jesus was the beginning of a new breed of men. Men who take a stand, even if no one else stands with them; vigilant men who faithfully protect; men who follow, then lead; and men who choose sacrifice in the name of love.<\/p>","startDate":"2016-04-12","endDate":"2016-05-01","trailerLink":null},"program":{"id":12789,"name":"05_24-25_14Program.pdf","title":"05_24-25_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_24-25_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5370,"containsAdultContent":"0","serviceId":"CsCEpeUQKRk","sourcePath":null,"source":{"id":12790,"name":"thenewman_05.mp4","title":"thenewman_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/thenewman_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"still":{"id":12791,"name":"thenewman_05_still.jpg","title":"thenewman_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/thenewman_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudImage"},"created":"2014-05-28T11:04:15-04:00","className":"MessageVideo"},"messageAudio":{"id":5371,"containsAdultContent":"0","serviceId":"protector","sourcePath":null,"source":{"id":12792,"name":"thenewman_05.mp3","title":"thenewman_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2014-06-05T12:41:54-04:00","className":"MessageAudio"},"created":"2014-06-05T12:41:54-04:00","className":"Message","number":5,"sequence":[{"id":3635,"date":"2014-04-26","number":1,"created":"2014-04-28 15:08:51","title":"5 Marks of a Man"},{"id":3636,"date":"2014-05-03","number":2,"created":"2014-05-05 10:41:12","title":"Minority Position"},{"id":3637,"date":"2014-05-10","number":3,"created":"2014-06-19 16:55:03","title":"Work"},{"id":3638,"date":"2014-05-17","number":4,"created":"2014-05-19 12:30:15","title":"Team Player"},{"id":3639,"date":"2014-05-24","number":5,"created":"2014-06-05 12:41:54","title":"Protector "},{"id":3645,"date":"2014-05-28","number":6,"created":"2014-06-10 15:50:41","title":"Last Wednesday - Oakley"},{"id":3646,"date":"2014-05-28","number":7,"created":"2014-06-19 17:11:24","title":"Last Wednesday - Florence"},{"id":3640,"date":"2016-08-30","number":8,"created":"2014-06-10 15:44:07","title":"Vision"}],"combinedTags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"relatedMessages":[{"id":3879,"title":"test1","description":"<p>test1<\/p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3640,"title":"Vision","description":"<p>Brian Tome talks about how the new man has vision.<\/p>","date":"2016-08-30","version":"1","series":205,"program":{"id":12793,"name":"05_31-06_01_14Program.pdf","title":"05_31-06_01_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_31-06_01_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5916,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"source":{"id":12841,"name":"071914_Florence.mp4","title":"071914_Florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/071914_Florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:28-05:00","className":"CloudFile"},"created":"2016-08-30T10:08:17-04:00","className":"MessageVideo"},"messageAudio":{"id":5373,"containsAdultContent":"0","serviceId":"vision","sourcePath":null,"source":{"id":12796,"name":"thenewman_06.mp3","title":"thenewman_06","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_06.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudFile"},"still":{"id":12797,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudImage"},"created":"2014-06-10T15:44:07-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2014-06-10T15:44:07-04:00","className":"Message"},{"id":3588,"title":"About the Small Places","description":"<p>Program - Week 5 God is_____.<\/p>","date":"2013-08-30","version":"1","series":194,"program":{"id":12574,"name":"08_10-11_13Program_LO.pdf","title":"08_10-11_13Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/08_10-11_13Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:54:41-05:00","className":"CloudFile"},"messageVideo":{"id":5274,"containsAdultContent":"0","serviceId":"rjncKOup7NE","sourcePath":null,"source":{"id":12572,"name":"god_is_05.mp4","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/god_is_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12573,"name":"god_is_05_still.jpg","title":"god_is_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/god_is_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"66970","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463415014}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:36:19-04:00","className":"MessageVideo"},"messageAudio":{"id":5273,"containsAdultContent":"0","serviceId":"about-the-small-places","sourcePath":null,"source":{"id":12570,"name":"god_is_05.mp3","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/god_is_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12571,"name":"190x110_GodIs.jpg","title":"190x110_GodIs","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GodIs.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"58329","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414758}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:29:04-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-08-12T13:39:27-04:00","className":"Message"},{"id":3579,"title":"Last Wednesday","description":"<p>Last Wednesday 6.26<\/p>","date":"2013-06-26","version":"1","series":193,"program":{"id":13991,"name":"pdf-sample-1.pdf","title":"pdf sample 1","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/pdf-sample-1.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"7945","cloudMetaJson":"{\"LastPut\":1459449748}","parent":812,"owner":33,"created":"2016-03-31T14:42:27-04:00","className":"CloudFile"},"messageVideo":{"id":5256,"containsAdultContent":"0","serviceId":"yfaCi6N3jLE","sourcePath":null,"source":{"id":12537,"name":"062613-LW.mp4","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/062613-LW.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12538,"name":"062613-LW-still.jpg","title":"062613-LW-still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/062613-LW-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"88181","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463414721}","parent":833,"owner":1,"derivedImages":[930],"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-27T15:43:23-04:00","className":"MessageVideo"},"messageAudio":{"id":5257,"containsAdultContent":"0","serviceId":"last-wednesday-6-26","sourcePath":null,"source":{"id":12539,"name":"062613-LW.mp3","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/062613-LW.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-27T15:40:39-04:00","className":"MessageAudio"},"speakers":[{"id":1,"title":"Brian Tome","messages":[3579,3265],"created":"2015-08-07T11:07:21-04:00","className":"Speaker"}],"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-27T15:40:39-04:00","className":"Message"},{"id":3578,"title":"Building Spiritual Maturity - Florence","description":"<p>Terry Phillips talks about building spiritual maturity.<\/p>","date":"2013-06-15","version":"1","series":193,"messageVideo":{"id":5255,"containsAdultContent":"0","serviceId":"2P8Xzyh-mGE","sourcePath":null,"source":{"id":12535,"name":"htbp_01_florence.mp4","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/htbp_01_florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12536,"name":"htbp_01_still_fl.jpg","title":"htbp_01_still_fl","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/htbp_01_still_fl.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-21T14:39:38-04:00","className":"MessageVideo"},"messageAudio":{"id":5254,"containsAdultContent":"0","serviceId":"building-spiritual-maturity-1","sourcePath":null,"source":{"id":12534,"name":"htbp_01_florence.mp3","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/htbp_01_florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:34-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-21T14:36:24-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-21T14:39:38-04:00","className":"Message"}]},{"id":3638,"title":"Team Player","description":"Chuck Mingo talks about how the new man is a team player.","date":"2014-05-17","version":"1","series":{"id":"205","title":"The New Man","description":"<p>Jesus was the beginning of a new breed of men. Men who take a stand, even if no one else stands with them; vigilant men who faithfully protect; men who follow, then lead; and men who choose sacrifice in the name of love.<\/p>","startDate":"2016-04-12","endDate":"2016-05-01","trailerLink":null},"program":{"id":12786,"name":"05_17-18_14Program_LO.pdf","title":"05_17-18_14Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_17-18_14Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5369,"containsAdultContent":"0","serviceId":"PYrZUJeHlz4","sourcePath":null,"source":{"id":12787,"name":"thenewman_04.mp4","title":"thenewman_04","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/thenewman_04.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"still":{"id":12788,"name":"thenewman_04_still.jpg","title":"thenewman_04_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/thenewman_04_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudImage"},"created":"2014-05-19T12:30:15-04:00","className":"MessageVideo"},"messageAudio":{"id":5368,"containsAdultContent":"0","serviceId":"team-player","sourcePath":null,"source":{"id":12784,"name":"thenewman_04.mp3","title":"thenewman_04","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_04.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:17-05:00","className":"CloudFile"},"still":{"id":12785,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:17-05:00","className":"CloudImage"},"created":"2014-05-19T12:23:10-04:00","className":"MessageAudio"},"created":"2014-05-19T12:30:15-04:00","className":"Message","number":4,"sequence":[{"id":3635,"date":"2014-04-26","number":1,"created":"2014-04-28 15:08:51","title":"5 Marks of a Man"},{"id":3636,"date":"2014-05-03","number":2,"created":"2014-05-05 10:41:12","title":"Minority Position"},{"id":3637,"date":"2014-05-10","number":3,"created":"2014-06-19 16:55:03","title":"Work"},{"id":3638,"date":"2014-05-17","number":4,"created":"2014-05-19 12:30:15","title":"Team Player"},{"id":3639,"date":"2014-05-24","number":5,"created":"2014-06-05 12:41:54","title":"Protector "},{"id":3645,"date":"2014-05-28","number":6,"created":"2014-06-10 15:50:41","title":"Last Wednesday - Oakley"},{"id":3646,"date":"2014-05-28","number":7,"created":"2014-06-19 17:11:24","title":"Last Wednesday - Florence"},{"id":3640,"date":"2016-08-30","number":8,"created":"2014-06-10 15:44:07","title":"Vision"}],"combinedTags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"relatedMessages":[{"id":3879,"title":"test1","description":"<p>test1<\/p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3640,"title":"Vision","description":"<p>Brian Tome talks about how the new man has vision.<\/p>","date":"2016-08-30","version":"1","series":205,"program":{"id":12793,"name":"05_31-06_01_14Program.pdf","title":"05_31-06_01_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_31-06_01_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5916,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"source":{"id":12841,"name":"071914_Florence.mp4","title":"071914_Florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/071914_Florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:28-05:00","className":"CloudFile"},"created":"2016-08-30T10:08:17-04:00","className":"MessageVideo"},"messageAudio":{"id":5373,"containsAdultContent":"0","serviceId":"vision","sourcePath":null,"source":{"id":12796,"name":"thenewman_06.mp3","title":"thenewman_06","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_06.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudFile"},"still":{"id":12797,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudImage"},"created":"2014-06-10T15:44:07-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2014-06-10T15:44:07-04:00","className":"Message"},{"id":3588,"title":"About the Small Places","description":"<p>Program - Week 5 God is_____.<\/p>","date":"2013-08-30","version":"1","series":194,"program":{"id":12574,"name":"08_10-11_13Program_LO.pdf","title":"08_10-11_13Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/08_10-11_13Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:54:41-05:00","className":"CloudFile"},"messageVideo":{"id":5274,"containsAdultContent":"0","serviceId":"rjncKOup7NE","sourcePath":null,"source":{"id":12572,"name":"god_is_05.mp4","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/god_is_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12573,"name":"god_is_05_still.jpg","title":"god_is_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/god_is_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"66970","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463415014}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:36:19-04:00","className":"MessageVideo"},"messageAudio":{"id":5273,"containsAdultContent":"0","serviceId":"about-the-small-places","sourcePath":null,"source":{"id":12570,"name":"god_is_05.mp3","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/god_is_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12571,"name":"190x110_GodIs.jpg","title":"190x110_GodIs","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GodIs.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"58329","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414758}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:29:04-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-08-12T13:39:27-04:00","className":"Message"},{"id":3579,"title":"Last Wednesday","description":"<p>Last Wednesday 6.26<\/p>","date":"2013-06-26","version":"1","series":193,"program":{"id":13991,"name":"pdf-sample-1.pdf","title":"pdf sample 1","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/pdf-sample-1.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"7945","cloudMetaJson":"{\"LastPut\":1459449748}","parent":812,"owner":33,"created":"2016-03-31T14:42:27-04:00","className":"CloudFile"},"messageVideo":{"id":5256,"containsAdultContent":"0","serviceId":"yfaCi6N3jLE","sourcePath":null,"source":{"id":12537,"name":"062613-LW.mp4","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/062613-LW.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12538,"name":"062613-LW-still.jpg","title":"062613-LW-still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/062613-LW-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"88181","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463414721}","parent":833,"owner":1,"derivedImages":[930],"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-27T15:43:23-04:00","className":"MessageVideo"},"messageAudio":{"id":5257,"containsAdultContent":"0","serviceId":"last-wednesday-6-26","sourcePath":null,"source":{"id":12539,"name":"062613-LW.mp3","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/062613-LW.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-27T15:40:39-04:00","className":"MessageAudio"},"speakers":[{"id":1,"title":"Brian Tome","messages":[3579,3265],"created":"2015-08-07T11:07:21-04:00","className":"Speaker"}],"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-27T15:40:39-04:00","className":"Message"},{"id":3578,"title":"Building Spiritual Maturity - Florence","description":"<p>Terry Phillips talks about building spiritual maturity.<\/p>","date":"2013-06-15","version":"1","series":193,"messageVideo":{"id":5255,"containsAdultContent":"0","serviceId":"2P8Xzyh-mGE","sourcePath":null,"source":{"id":12535,"name":"htbp_01_florence.mp4","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/htbp_01_florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12536,"name":"htbp_01_still_fl.jpg","title":"htbp_01_still_fl","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/htbp_01_still_fl.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-21T14:39:38-04:00","className":"MessageVideo"},"messageAudio":{"id":5254,"containsAdultContent":"0","serviceId":"building-spiritual-maturity-1","sourcePath":null,"source":{"id":12534,"name":"htbp_01_florence.mp3","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/htbp_01_florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:34-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-21T14:36:24-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-21T14:39:38-04:00","className":"Message"}]}]};
      let fixture = new CMSService(http);
      
      fixture.getMessages('');
      httpBackend.expectGET(`${endpoint}/messages?`).respond(200, messages);
      httpBackend.flush();
    })

    it('should use HTTP to obtain most recent messages', () => {
      let messages = {"messages":[{"id":3646,"title":"Last Wednesday - Florence","description":"Last Wednesday - Florence","date":"2014-05-28","version":"1","series":{"id":"205","title":"The New Man","description":"<p>Jesus was the beginning of a new breed of men. Men who take a stand, even if no one else stands with them; vigilant men who faithfully protect; men who follow, then lead; and men who choose sacrifice in the name of love.<\/p>","startDate":"2016-04-12","endDate":"2016-05-01","trailerLink":null},"messageVideo":{"id":5918,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"still":{"id":13607,"name":"082813_LW_Florence_Still.jpg","title":"082813_LW_Florence_Still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/stills\/082813_LW_Florence_Still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":817,"owner":1,"derivedImages":[690],"created":"2013-08-30T10:55:17-04:00","className":"CloudImage"},"created":"2016-08-30T11:58:39-04:00","className":"MessageVideo"},"messageAudio":{"id":5384,"containsAdultContent":"0","serviceId":"last-wednesday-may-1","sourcePath":null,"source":{"id":12821,"name":"022614-lw-florence.mp3","title":"022614-lw-florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/022614-lw-florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:24-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2014-06-10T16:52:14-04:00","className":"MessageAudio"},"created":"2014-06-19T17:11:24-04:00","className":"Message","number":7,"sequence":[{"id":3635,"date":"2014-04-26","number":1,"created":"2014-04-28 15:08:51","title":"5 Marks of a Man"},{"id":3636,"date":"2014-05-03","number":2,"created":"2014-05-05 10:41:12","title":"Minority Position"},{"id":3637,"date":"2014-05-10","number":3,"created":"2014-06-19 16:55:03","title":"Work"},{"id":3638,"date":"2014-05-17","number":4,"created":"2014-05-19 12:30:15","title":"Team Player"},{"id":3639,"date":"2014-05-24","number":5,"created":"2014-06-05 12:41:54","title":"Protector "},{"id":3645,"date":"2014-05-28","number":6,"created":"2014-06-10 15:50:41","title":"Last Wednesday - Oakley"},{"id":3646,"date":"2014-05-28","number":7,"created":"2014-06-19 17:11:24","title":"Last Wednesday - Florence"},{"id":3640,"date":"2016-08-30","number":8,"created":"2014-06-10 15:44:07","title":"Vision"}],"combinedTags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"relatedMessages":[{"id":3879,"title":"test1","description":"<p>test1<\/p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3640,"title":"Vision","description":"<p>Brian Tome talks about how the new man has vision.<\/p>","date":"2016-08-30","version":"1","series":205,"program":{"id":12793,"name":"05_31-06_01_14Program.pdf","title":"05_31-06_01_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_31-06_01_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5916,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"source":{"id":12841,"name":"071914_Florence.mp4","title":"071914_Florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/071914_Florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:28-05:00","className":"CloudFile"},"created":"2016-08-30T10:08:17-04:00","className":"MessageVideo"},"messageAudio":{"id":5373,"containsAdultContent":"0","serviceId":"vision","sourcePath":null,"source":{"id":12796,"name":"thenewman_06.mp3","title":"thenewman_06","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_06.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudFile"},"still":{"id":12797,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudImage"},"created":"2014-06-10T15:44:07-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2014-06-10T15:44:07-04:00","className":"Message"},{"id":3588,"title":"About the Small Places","description":"<p>Program - Week 5 God is_____.<\/p>","date":"2013-08-30","version":"1","series":194,"program":{"id":12574,"name":"08_10-11_13Program_LO.pdf","title":"08_10-11_13Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/08_10-11_13Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:54:41-05:00","className":"CloudFile"},"messageVideo":{"id":5274,"containsAdultContent":"0","serviceId":"rjncKOup7NE","sourcePath":null,"source":{"id":12572,"name":"god_is_05.mp4","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/god_is_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12573,"name":"god_is_05_still.jpg","title":"god_is_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/god_is_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"66970","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463415014}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:36:19-04:00","className":"MessageVideo"},"messageAudio":{"id":5273,"containsAdultContent":"0","serviceId":"about-the-small-places","sourcePath":null,"source":{"id":12570,"name":"god_is_05.mp3","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/god_is_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12571,"name":"190x110_GodIs.jpg","title":"190x110_GodIs","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GodIs.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"58329","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414758}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:29:04-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-08-12T13:39:27-04:00","className":"Message"},{"id":3579,"title":"Last Wednesday","description":"<p>Last Wednesday 6.26<\/p>","date":"2013-06-26","version":"1","series":193,"program":{"id":13991,"name":"pdf-sample-1.pdf","title":"pdf sample 1","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/pdf-sample-1.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"7945","cloudMetaJson":"{\"LastPut\":1459449748}","parent":812,"owner":33,"created":"2016-03-31T14:42:27-04:00","className":"CloudFile"},"messageVideo":{"id":5256,"containsAdultContent":"0","serviceId":"yfaCi6N3jLE","sourcePath":null,"source":{"id":12537,"name":"062613-LW.mp4","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/062613-LW.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12538,"name":"062613-LW-still.jpg","title":"062613-LW-still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/062613-LW-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"88181","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463414721}","parent":833,"owner":1,"derivedImages":[930],"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-27T15:43:23-04:00","className":"MessageVideo"},"messageAudio":{"id":5257,"containsAdultContent":"0","serviceId":"last-wednesday-6-26","sourcePath":null,"source":{"id":12539,"name":"062613-LW.mp3","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/062613-LW.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-27T15:40:39-04:00","className":"MessageAudio"},"speakers":[{"id":1,"title":"Brian Tome","messages":[3579,3265],"created":"2015-08-07T11:07:21-04:00","className":"Speaker"}],"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-27T15:40:39-04:00","className":"Message"},{"id":3578,"title":"Building Spiritual Maturity - Florence","description":"<p>Terry Phillips talks about building spiritual maturity.<\/p>","date":"2013-06-15","version":"1","series":193,"messageVideo":{"id":5255,"containsAdultContent":"0","serviceId":"2P8Xzyh-mGE","sourcePath":null,"source":{"id":12535,"name":"htbp_01_florence.mp4","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/htbp_01_florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12536,"name":"htbp_01_still_fl.jpg","title":"htbp_01_still_fl","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/htbp_01_still_fl.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-21T14:39:38-04:00","className":"MessageVideo"},"messageAudio":{"id":5254,"containsAdultContent":"0","serviceId":"building-spiritual-maturity-1","sourcePath":null,"source":{"id":12534,"name":"htbp_01_florence.mp3","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/htbp_01_florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:34-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-21T14:36:24-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-21T14:39:38-04:00","className":"Message"}]},{"id":3645,"title":"Last Wednesday - Oakley","description":"Last Wednesday - Oakley","date":"2014-05-28","version":"1","series":{"id":"205","title":"The New Man","description":"<p>Jesus was the beginning of a new breed of men. Men who take a stand, even if no one else stands with them; vigilant men who faithfully protect; men who follow, then lead; and men who choose sacrifice in the name of love.<\/p>","startDate":"2016-04-12","endDate":"2016-05-01","trailerLink":null},"messageVideo":{"id":5383,"containsAdultContent":"0","serviceId":"c41wQrbdDhM","sourcePath":null,"source":{"id":12819,"name":"052814-lw-oakley.mp4","title":"052814-lw-oakley","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/052814-lw-oakley.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:23-05:00","className":"CloudFile"},"still":{"id":12820,"name":"last-wednesday.jpg","title":"last-wednesday","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/last-wednesday.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:23-05:00","className":"CloudImage"},"created":"2014-06-10T15:50:41-04:00","className":"MessageVideo"},"messageAudio":{"id":5382,"containsAdultContent":"0","serviceId":"last-wednesday-may","sourcePath":null,"source":{"id":12818,"name":"052814-lw-oakley.mp3","title":"052814-lw-oakley","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/052814-lw-oakley.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:23-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2014-06-10T15:39:36-04:00","className":"MessageAudio"},"created":"2014-06-10T15:50:41-04:00","className":"Message","number":6,"sequence":[{"id":3635,"date":"2014-04-26","number":1,"created":"2014-04-28 15:08:51","title":"5 Marks of a Man"},{"id":3636,"date":"2014-05-03","number":2,"created":"2014-05-05 10:41:12","title":"Minority Position"},{"id":3637,"date":"2014-05-10","number":3,"created":"2014-06-19 16:55:03","title":"Work"},{"id":3638,"date":"2014-05-17","number":4,"created":"2014-05-19 12:30:15","title":"Team Player"},{"id":3639,"date":"2014-05-24","number":5,"created":"2014-06-05 12:41:54","title":"Protector "},{"id":3645,"date":"2014-05-28","number":6,"created":"2014-06-10 15:50:41","title":"Last Wednesday - Oakley"},{"id":3646,"date":"2014-05-28","number":7,"created":"2014-06-19 17:11:24","title":"Last Wednesday - Florence"},{"id":3640,"date":"2016-08-30","number":8,"created":"2014-06-10 15:44:07","title":"Vision"}],"combinedTags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"relatedMessages":[{"id":3879,"title":"test1","description":"<p>test1<\/p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3640,"title":"Vision","description":"<p>Brian Tome talks about how the new man has vision.<\/p>","date":"2016-08-30","version":"1","series":205,"program":{"id":12793,"name":"05_31-06_01_14Program.pdf","title":"05_31-06_01_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_31-06_01_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5916,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"source":{"id":12841,"name":"071914_Florence.mp4","title":"071914_Florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/071914_Florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:28-05:00","className":"CloudFile"},"created":"2016-08-30T10:08:17-04:00","className":"MessageVideo"},"messageAudio":{"id":5373,"containsAdultContent":"0","serviceId":"vision","sourcePath":null,"source":{"id":12796,"name":"thenewman_06.mp3","title":"thenewman_06","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_06.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudFile"},"still":{"id":12797,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudImage"},"created":"2014-06-10T15:44:07-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2014-06-10T15:44:07-04:00","className":"Message"},{"id":3588,"title":"About the Small Places","description":"<p>Program - Week 5 God is_____.<\/p>","date":"2013-08-30","version":"1","series":194,"program":{"id":12574,"name":"08_10-11_13Program_LO.pdf","title":"08_10-11_13Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/08_10-11_13Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:54:41-05:00","className":"CloudFile"},"messageVideo":{"id":5274,"containsAdultContent":"0","serviceId":"rjncKOup7NE","sourcePath":null,"source":{"id":12572,"name":"god_is_05.mp4","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/god_is_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12573,"name":"god_is_05_still.jpg","title":"god_is_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/god_is_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"66970","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463415014}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:36:19-04:00","className":"MessageVideo"},"messageAudio":{"id":5273,"containsAdultContent":"0","serviceId":"about-the-small-places","sourcePath":null,"source":{"id":12570,"name":"god_is_05.mp3","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/god_is_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12571,"name":"190x110_GodIs.jpg","title":"190x110_GodIs","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GodIs.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"58329","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414758}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:29:04-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-08-12T13:39:27-04:00","className":"Message"},{"id":3579,"title":"Last Wednesday","description":"<p>Last Wednesday 6.26<\/p>","date":"2013-06-26","version":"1","series":193,"program":{"id":13991,"name":"pdf-sample-1.pdf","title":"pdf sample 1","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/pdf-sample-1.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"7945","cloudMetaJson":"{\"LastPut\":1459449748}","parent":812,"owner":33,"created":"2016-03-31T14:42:27-04:00","className":"CloudFile"},"messageVideo":{"id":5256,"containsAdultContent":"0","serviceId":"yfaCi6N3jLE","sourcePath":null,"source":{"id":12537,"name":"062613-LW.mp4","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/062613-LW.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12538,"name":"062613-LW-still.jpg","title":"062613-LW-still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/062613-LW-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"88181","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463414721}","parent":833,"owner":1,"derivedImages":[930],"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-27T15:43:23-04:00","className":"MessageVideo"},"messageAudio":{"id":5257,"containsAdultContent":"0","serviceId":"last-wednesday-6-26","sourcePath":null,"source":{"id":12539,"name":"062613-LW.mp3","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/062613-LW.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-27T15:40:39-04:00","className":"MessageAudio"},"speakers":[{"id":1,"title":"Brian Tome","messages":[3579,3265],"created":"2015-08-07T11:07:21-04:00","className":"Speaker"}],"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-27T15:40:39-04:00","className":"Message"},{"id":3578,"title":"Building Spiritual Maturity - Florence","description":"<p>Terry Phillips talks about building spiritual maturity.<\/p>","date":"2013-06-15","version":"1","series":193,"messageVideo":{"id":5255,"containsAdultContent":"0","serviceId":"2P8Xzyh-mGE","sourcePath":null,"source":{"id":12535,"name":"htbp_01_florence.mp4","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/htbp_01_florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12536,"name":"htbp_01_still_fl.jpg","title":"htbp_01_still_fl","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/htbp_01_still_fl.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-21T14:39:38-04:00","className":"MessageVideo"},"messageAudio":{"id":5254,"containsAdultContent":"0","serviceId":"building-spiritual-maturity-1","sourcePath":null,"source":{"id":12534,"name":"htbp_01_florence.mp3","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/htbp_01_florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:34-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-21T14:36:24-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-21T14:39:38-04:00","className":"Message"}]},{"id":3639,"title":"Protector ","description":"Brian Tome talks about how the new man is a protector.","date":"2014-05-24","version":"1","series":{"id":"205","title":"The New Man","description":"<p>Jesus was the beginning of a new breed of men. Men who take a stand, even if no one else stands with them; vigilant men who faithfully protect; men who follow, then lead; and men who choose sacrifice in the name of love.<\/p>","startDate":"2016-04-12","endDate":"2016-05-01","trailerLink":null},"program":{"id":12789,"name":"05_24-25_14Program.pdf","title":"05_24-25_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_24-25_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5370,"containsAdultContent":"0","serviceId":"CsCEpeUQKRk","sourcePath":null,"source":{"id":12790,"name":"thenewman_05.mp4","title":"thenewman_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/thenewman_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"still":{"id":12791,"name":"thenewman_05_still.jpg","title":"thenewman_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/thenewman_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudImage"},"created":"2014-05-28T11:04:15-04:00","className":"MessageVideo"},"messageAudio":{"id":5371,"containsAdultContent":"0","serviceId":"protector","sourcePath":null,"source":{"id":12792,"name":"thenewman_05.mp3","title":"thenewman_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2014-06-05T12:41:54-04:00","className":"MessageAudio"},"created":"2014-06-05T12:41:54-04:00","className":"Message","number":5,"sequence":[{"id":3635,"date":"2014-04-26","number":1,"created":"2014-04-28 15:08:51","title":"5 Marks of a Man"},{"id":3636,"date":"2014-05-03","number":2,"created":"2014-05-05 10:41:12","title":"Minority Position"},{"id":3637,"date":"2014-05-10","number":3,"created":"2014-06-19 16:55:03","title":"Work"},{"id":3638,"date":"2014-05-17","number":4,"created":"2014-05-19 12:30:15","title":"Team Player"},{"id":3639,"date":"2014-05-24","number":5,"created":"2014-06-05 12:41:54","title":"Protector "},{"id":3645,"date":"2014-05-28","number":6,"created":"2014-06-10 15:50:41","title":"Last Wednesday - Oakley"},{"id":3646,"date":"2014-05-28","number":7,"created":"2014-06-19 17:11:24","title":"Last Wednesday - Florence"},{"id":3640,"date":"2016-08-30","number":8,"created":"2014-06-10 15:44:07","title":"Vision"}],"combinedTags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"relatedMessages":[{"id":3879,"title":"test1","description":"<p>test1<\/p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3640,"title":"Vision","description":"<p>Brian Tome talks about how the new man has vision.<\/p>","date":"2016-08-30","version":"1","series":205,"program":{"id":12793,"name":"05_31-06_01_14Program.pdf","title":"05_31-06_01_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_31-06_01_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5916,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"source":{"id":12841,"name":"071914_Florence.mp4","title":"071914_Florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/071914_Florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:28-05:00","className":"CloudFile"},"created":"2016-08-30T10:08:17-04:00","className":"MessageVideo"},"messageAudio":{"id":5373,"containsAdultContent":"0","serviceId":"vision","sourcePath":null,"source":{"id":12796,"name":"thenewman_06.mp3","title":"thenewman_06","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_06.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudFile"},"still":{"id":12797,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudImage"},"created":"2014-06-10T15:44:07-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2014-06-10T15:44:07-04:00","className":"Message"},{"id":3588,"title":"About the Small Places","description":"<p>Program - Week 5 God is_____.<\/p>","date":"2013-08-30","version":"1","series":194,"program":{"id":12574,"name":"08_10-11_13Program_LO.pdf","title":"08_10-11_13Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/08_10-11_13Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:54:41-05:00","className":"CloudFile"},"messageVideo":{"id":5274,"containsAdultContent":"0","serviceId":"rjncKOup7NE","sourcePath":null,"source":{"id":12572,"name":"god_is_05.mp4","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/god_is_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12573,"name":"god_is_05_still.jpg","title":"god_is_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/god_is_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"66970","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463415014}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:36:19-04:00","className":"MessageVideo"},"messageAudio":{"id":5273,"containsAdultContent":"0","serviceId":"about-the-small-places","sourcePath":null,"source":{"id":12570,"name":"god_is_05.mp3","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/god_is_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12571,"name":"190x110_GodIs.jpg","title":"190x110_GodIs","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GodIs.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"58329","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414758}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:29:04-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-08-12T13:39:27-04:00","className":"Message"},{"id":3579,"title":"Last Wednesday","description":"<p>Last Wednesday 6.26<\/p>","date":"2013-06-26","version":"1","series":193,"program":{"id":13991,"name":"pdf-sample-1.pdf","title":"pdf sample 1","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/pdf-sample-1.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"7945","cloudMetaJson":"{\"LastPut\":1459449748}","parent":812,"owner":33,"created":"2016-03-31T14:42:27-04:00","className":"CloudFile"},"messageVideo":{"id":5256,"containsAdultContent":"0","serviceId":"yfaCi6N3jLE","sourcePath":null,"source":{"id":12537,"name":"062613-LW.mp4","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/062613-LW.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12538,"name":"062613-LW-still.jpg","title":"062613-LW-still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/062613-LW-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"88181","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463414721}","parent":833,"owner":1,"derivedImages":[930],"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-27T15:43:23-04:00","className":"MessageVideo"},"messageAudio":{"id":5257,"containsAdultContent":"0","serviceId":"last-wednesday-6-26","sourcePath":null,"source":{"id":12539,"name":"062613-LW.mp3","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/062613-LW.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-27T15:40:39-04:00","className":"MessageAudio"},"speakers":[{"id":1,"title":"Brian Tome","messages":[3579,3265],"created":"2015-08-07T11:07:21-04:00","className":"Speaker"}],"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-27T15:40:39-04:00","className":"Message"},{"id":3578,"title":"Building Spiritual Maturity - Florence","description":"<p>Terry Phillips talks about building spiritual maturity.<\/p>","date":"2013-06-15","version":"1","series":193,"messageVideo":{"id":5255,"containsAdultContent":"0","serviceId":"2P8Xzyh-mGE","sourcePath":null,"source":{"id":12535,"name":"htbp_01_florence.mp4","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/htbp_01_florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12536,"name":"htbp_01_still_fl.jpg","title":"htbp_01_still_fl","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/htbp_01_still_fl.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-21T14:39:38-04:00","className":"MessageVideo"},"messageAudio":{"id":5254,"containsAdultContent":"0","serviceId":"building-spiritual-maturity-1","sourcePath":null,"source":{"id":12534,"name":"htbp_01_florence.mp3","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/htbp_01_florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:34-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-21T14:36:24-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-21T14:39:38-04:00","className":"Message"}]},{"id":3638,"title":"Team Player","description":"Chuck Mingo talks about how the new man is a team player.","date":"2014-05-17","version":"1","series":{"id":"205","title":"The New Man","description":"<p>Jesus was the beginning of a new breed of men. Men who take a stand, even if no one else stands with them; vigilant men who faithfully protect; men who follow, then lead; and men who choose sacrifice in the name of love.<\/p>","startDate":"2016-04-12","endDate":"2016-05-01","trailerLink":null},"program":{"id":12786,"name":"05_17-18_14Program_LO.pdf","title":"05_17-18_14Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_17-18_14Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5369,"containsAdultContent":"0","serviceId":"PYrZUJeHlz4","sourcePath":null,"source":{"id":12787,"name":"thenewman_04.mp4","title":"thenewman_04","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/thenewman_04.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"still":{"id":12788,"name":"thenewman_04_still.jpg","title":"thenewman_04_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/thenewman_04_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudImage"},"created":"2014-05-19T12:30:15-04:00","className":"MessageVideo"},"messageAudio":{"id":5368,"containsAdultContent":"0","serviceId":"team-player","sourcePath":null,"source":{"id":12784,"name":"thenewman_04.mp3","title":"thenewman_04","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_04.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:17-05:00","className":"CloudFile"},"still":{"id":12785,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:17-05:00","className":"CloudImage"},"created":"2014-05-19T12:23:10-04:00","className":"MessageAudio"},"created":"2014-05-19T12:30:15-04:00","className":"Message","number":4,"sequence":[{"id":3635,"date":"2014-04-26","number":1,"created":"2014-04-28 15:08:51","title":"5 Marks of a Man"},{"id":3636,"date":"2014-05-03","number":2,"created":"2014-05-05 10:41:12","title":"Minority Position"},{"id":3637,"date":"2014-05-10","number":3,"created":"2014-06-19 16:55:03","title":"Work"},{"id":3638,"date":"2014-05-17","number":4,"created":"2014-05-19 12:30:15","title":"Team Player"},{"id":3639,"date":"2014-05-24","number":5,"created":"2014-06-05 12:41:54","title":"Protector "},{"id":3645,"date":"2014-05-28","number":6,"created":"2014-06-10 15:50:41","title":"Last Wednesday - Oakley"},{"id":3646,"date":"2014-05-28","number":7,"created":"2014-06-19 17:11:24","title":"Last Wednesday - Florence"},{"id":3640,"date":"2016-08-30","number":8,"created":"2014-06-10 15:44:07","title":"Vision"}],"combinedTags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"relatedMessages":[{"id":3879,"title":"test1","description":"<p>test1<\/p>","date":"2016-09-14","version":"1","series":318,"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2016-09-14T17:03:48-04:00","className":"Message"},{"id":3640,"title":"Vision","description":"<p>Brian Tome talks about how the new man has vision.<\/p>","date":"2016-08-30","version":"1","series":205,"program":{"id":12793,"name":"05_31-06_01_14Program.pdf","title":"05_31-06_01_14Program","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/05_31-06_01_14Program.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:55:18-05:00","className":"CloudFile"},"messageVideo":{"id":5916,"containsAdultContent":"0","serviceId":"123","sourcePath":null,"source":{"id":12841,"name":"071914_Florence.mp4","title":"071914_Florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/071914_Florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:55:28-05:00","className":"CloudFile"},"created":"2016-08-30T10:08:17-04:00","className":"MessageVideo"},"messageAudio":{"id":5373,"containsAdultContent":"0","serviceId":"vision","sourcePath":null,"source":{"id":12796,"name":"thenewman_06.mp3","title":"thenewman_06","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/thenewman_06.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudFile"},"still":{"id":12797,"name":"TheNewMan_190x110.jpg","title":"TheNewMan_190x110","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/TheNewMan_190x110.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:55:19-05:00","className":"CloudImage"},"created":"2014-06-10T15:44:07-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2014-06-10T15:44:07-04:00","className":"Message"},{"id":3588,"title":"About the Small Places","description":"<p>Program - Week 5 God is_____.<\/p>","date":"2013-08-30","version":"1","series":194,"program":{"id":12574,"name":"08_10-11_13Program_LO.pdf","title":"08_10-11_13Program_LO","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/08_10-11_13Program_LO.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":812,"owner":1,"created":"2016-01-25T19:54:41-05:00","className":"CloudFile"},"messageVideo":{"id":5274,"containsAdultContent":"0","serviceId":"rjncKOup7NE","sourcePath":null,"source":{"id":12572,"name":"god_is_05.mp4","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/god_is_05.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12573,"name":"god_is_05_still.jpg","title":"god_is_05_still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/god_is_05_still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"66970","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463415014}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:36:19-04:00","className":"MessageVideo"},"messageAudio":{"id":5273,"containsAdultContent":"0","serviceId":"about-the-small-places","sourcePath":null,"source":{"id":12570,"name":"god_is_05.mp3","title":"god_is_05","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/god_is_05.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudFile"},"still":{"id":12571,"name":"190x110_GodIs.jpg","title":"190x110_GodIs","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/190x110_GodIs.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"58329","cloudMetaJson":"{\"Dimensions\":\"190x110\",\"LastPut\":1463414758}","parent":833,"owner":1,"created":"2016-01-25T19:54:40-05:00","className":"CloudImage"},"created":"2013-08-12T13:29:04-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-08-12T13:39:27-04:00","className":"Message"},{"id":3579,"title":"Last Wednesday","description":"<p>Last Wednesday 6.26<\/p>","date":"2013-06-26","version":"1","series":193,"program":{"id":13991,"name":"pdf-sample-1.pdf","title":"pdf sample 1","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/documents\/pdf-sample-1.pdf","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"7945","cloudMetaJson":"{\"LastPut\":1459449748}","parent":812,"owner":33,"created":"2016-03-31T14:42:27-04:00","className":"CloudFile"},"messageVideo":{"id":5256,"containsAdultContent":"0","serviceId":"yfaCi6N3jLE","sourcePath":null,"source":{"id":12537,"name":"062613-LW.mp4","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/062613-LW.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12538,"name":"062613-LW-still.jpg","title":"062613-LW-still","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/062613-LW-still.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"88181","cloudMetaJson":"{\"Dimensions\":\"640x360\",\"LastPut\":1463414721}","parent":833,"owner":1,"derivedImages":[930],"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-27T15:43:23-04:00","className":"MessageVideo"},"messageAudio":{"id":5257,"containsAdultContent":"0","serviceId":"last-wednesday-6-26","sourcePath":null,"source":{"id":12539,"name":"062613-LW.mp3","title":"062613-LW","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/062613-LW.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-27T15:40:39-04:00","className":"MessageAudio"},"speakers":[{"id":1,"title":"Brian Tome","messages":[3579,3265],"created":"2015-08-07T11:07:21-04:00","className":"Speaker"}],"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-27T15:40:39-04:00","className":"Message"},{"id":3578,"title":"Building Spiritual Maturity - Florence","description":"<p>Terry Phillips talks about building spiritual maturity.<\/p>","date":"2013-06-15","version":"1","series":193,"messageVideo":{"id":5255,"containsAdultContent":"0","serviceId":"2P8Xzyh-mGE","sourcePath":null,"source":{"id":12535,"name":"htbp_01_florence.mp4","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/video\/htbp_01_florence.mp4","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":827,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudFile"},"still":{"id":12536,"name":"htbp_01_still_fl.jpg","title":"htbp_01_still_fl","filename":"https:\/\/crds-cms-uploads.imgix.net\/media\/messages\/stills\/htbp_01_still_fl.jpg","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":833,"owner":1,"created":"2016-01-25T19:54:35-05:00","className":"CloudImage"},"created":"2013-06-21T14:39:38-04:00","className":"MessageVideo"},"messageAudio":{"id":5254,"containsAdultContent":"0","serviceId":"building-spiritual-maturity-1","sourcePath":null,"source":{"id":12534,"name":"htbp_01_florence.mp3","title":"htbp_01_florence","filename":"https:\/\/s3.amazonaws.com\/crds-cms-uploads\/media\/messages\/audio\/htbp_01_florence.mp3","content":null,"showInSearch":"1","cloudStatus":"Live","cloudSize":"0","cloudMetaJson":null,"parent":808,"owner":1,"created":"2016-01-25T19:54:34-05:00","className":"CloudFile"},"still":{"id":0,"name":"assets","title":null,"filename":"\/assets\/","content":null,"showInSearch":1,"cloudStatus":null,"cloudSize":null,"cloudMetaJson":null,"className":"Image"},"created":"2013-06-21T14:36:24-04:00","className":"MessageAudio"},"tags":[{"id":33,"title":"all","series":[194,205,5],"messages":[3579,3578,3588,3640,3879],"created":"2015-09-23T11:52:06-04:00","className":"Tag"}],"created":"2013-06-21T14:39:38-04:00","className":"Message"}]}]}
      let limit = 4
      let fixture = new CMSService(http);
      
      fixture.getRecentMessages(limit).then(response => {
        expect(response.length).toBe(limit);
      });
      httpBackend.expectGET(`${endpoint}/messages?date__LessThanOrEqual=${todaysDate}&date__sort=DESC&ID__sort=DESC&SeriesID__GreaterThan=0&__limit[]=${limit}`).respond(200, messages);
      httpBackend.flush();
    })
  })

})
